﻿'Title:                 JRL-3000L frequency tracking with IC-7700
'Author:                H.NAMVA
'Original issued on：   2013.11.11 ver 1.0
'Last updated on:       2013.11.11
Public Class Form1
    'variables for IC-7700
    Private RxBuf(1024) As Byte
    Private RxBufLength As Integer
    Private Frequency As Integer = 0
    'variables for JRL-3000F
    Private RxBufJRC(256) As Byte
    Private RxBufLenJRC As Integer
    Private JRCready As Boolean = False
    Private PrevFreq As Integer = 0

    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim inqFreq() As Byte = {&HFE, &HFE, &H74, &H0, &H3, &HFD}
        Dim availableSerialPorts As System.Collections.ObjectModel.ReadOnlyCollection(Of String) = My.Computer.Ports.SerialPortNames
        Debug.Print(Now & "program started" & vbCrLf)
        Me.JRCready = False
        Me.Frequency = 0
        Me.PrevFreq = 0
        LabelJRL3000.Text = ""
        Try
            If availableSerialPorts.Contains(COMPortJRC.PortName) Then
                If COMPortJRC.IsOpen Then
                    MessageBox.Show(COMPortJRC.PortName & ", for JRL-3000F, is already OPENED", "error", _
                                    MessageBoxButtons.OK, MessageBoxIcon.Error)
                    Me.Close()
                Else
                    COMPortJRC.Open()
                End If
            Else
                MsgBox(COMPortJRC.PortName & " is not available")
                Me.Close()
            End If
        Catch ex As Exception
            Debug.Print(ex.Message & " in Form1_Load")
            Me.Close()
        End Try

        Try
            If availableSerialPorts.Contains(COMportICOM.PortName) Then
                If COMportICOM.IsOpen Then
                    MessageBox.Show(COMportICOM.PortName & ", for IC-7700, is already OPENED", "error", _
                                    MessageBoxButtons.OK, MessageBoxIcon.Error)
                    COMPortJRC.Close()
                    Me.Close()
                Else
                    COMportICOM.Open()
                    COMportICOM.Write(inqFreq, 0, 6)
                End If
            Else
                MsgBox(COMportICOM.PortName & " is not available")
                COMPortJRC.Close()
                Me.Close()
            End If
        Catch ex As Exception
            MsgBox(ex.Message & " in Form1_Load")
            COMPortJRC.Close()
            Me.Close()
        End Try
    End Sub

    Private Sub COMportICOM_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles COMportICOM.DataReceived
        Dim len As Integer
        Dim AddLineIntoTxbRx As New AddLineDelegate(AddressOf AddLine)

        Try
            len = COMportICOM.Read(Me.RxBuf, Me.RxBufLength, Me.RxBuf.Length - Me.RxBufLength)
            If len = 0 Then
                Return
            End If
            Me.RxBufLength += len
            If Me.RxBuf(RxBufLength - 1) = &HFD Then
                TxbRx.Invoke(AddLineIntoTxbRx)
            End If
        Catch ex As Exception
            Debug.Print(ex.Message & vbCrLf & "RxBufLength = " & RxBufLength.ToString)
        End Try
    End Sub

    Private Function ByteToHex(ByVal bin As Byte) As String
        Dim hex As String = "0123456789ABCDEF"

        ByteToHex = hex.Substring((bin >> 4), 1) & hex.Substring((bin And &HF), 1)
    End Function

    Delegate Sub AddLineDelegate()

    Private Sub GetFreqData(ByVal str As String)
        Dim FreqStr As String = ""
        Try

            If str.Substring(0, 4) <> "FEFE" Then
                'プリアンブルが一致しない場合はパス
                Exit Sub
            End If
            If str.Substring(6, 2) <> "74" Then
                'IC7700から発せられたメッセージ以外はパス
                Exit Sub
            End If
            If str.Substring(8, 2) = "00" Or str.Substring(8, 2) = "03" Then
                '周波数データ
                FreqStr = str.Substring(18, 2) & str.Substring(16, 2) & str.Substring(14, 2) _
                    & str.Substring(12, 2) & str.Substring(10, 2)
                Me.Frequency = Integer.Parse(FreqStr)
                LabelFreq.Text = Me.Frequency.ToString("#,0") & "Hz"

            End If
        Catch ex As Exception
            Debug.Print(ex.Message & "in GetFreqData")
        End Try
    End Sub

    Private Sub AddLine()
        Dim str As String = ""
        Dim temp As String = ""

        For i As Integer = 0 To RxBufLength - 1
            str &= ByteToHex(RxBuf(i))
        Next
        RxBufLength = 0
        TxbRx.AppendText(str & vbCrLf)
        GetFreqData(str)

        If Len(TxbRx.Text) > (TxbRx.MaxLength - 1000) Then
            'サイズが大きくなり過ぎたら、データを半分消去する
            temp = Mid(TxbRx.Text, TxbRx.MaxLength / 2, Len(TxbRx.Text))
            TxbRx.Text = temp
        End If
    End Sub

    Private Function HexToByte(ByVal hex2digit As String) As Integer
        Dim hexDigit As String = "0123456789ABCDEF"
        Dim value As Integer
        HexToByte = -1
        value = hexDigit.IndexOf(hex2digit(0))
        If value < 0 Then
            Exit Function
        End If
        HexToByte = value * 16
        value = hexDigit.IndexOf(hex2digit(1))
        If value < 0 Then
            Exit Function
        End If
        HexToByte += value
    End Function

    Private Function HexStrToByteArray(ByVal strHex As String, ByRef arrayByte As Byte(), ByRef lenArray As Integer) As Integer
        Dim byteValue As Integer
        HexStrToByteArray = 0
        lenArray = 0
        While strHex.Length > 0
            byteValue = HexToByte(strHex.Substring(0, 2))
            If byteValue < 0 Then
                HexStrToByteArray = -1
                Exit Function
            End If
            arrayByte(lenArray) = byteValue
            lenArray += 1
            HexStrToByteArray += 1
            strHex = strHex.Substring(2)
        End While
    End Function


    Private Sub COMPortJRC_DataReceived(ByVal sender As System.Object, ByVal e As System.IO.Ports.SerialDataReceivedEventArgs) Handles COMPortJRC.DataReceived
        Dim len As Integer
        Try
            len = COMPortJRC.Read(Me.RxBufJRC, Me.RxBufLenJRC, 1)
            If len = 0 Then
                Return
            End If
            If Me.RxBufJRC(Me.RxBufLenJRC) = &HD And Me.RxBufLenJRC > 0 Then
                If Me.RxBufJRC(Me.RxBufLenJRC - 1) = &H23 Then
                    '接続確認応答
                    Me.JRCready = True
                    Me.RxBufLenJRC = 0
                End If
            Else
                Me.RxBufLenJRC += 1
                If Me.RxBufLenJRC > 10 Then RxBufLenJRC = 0
            End If
        Catch ex As Exception
            Debug.Print(ex.Message & " in COMPortJRC_DataRecived")
            Me.RxBufLenJRC = 0 'clearRX buffer for JRL-3000F
        End Try
    End Sub


    Private Sub Timer1_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer1.Tick
        Dim TxBufJRC_connect() As Byte = {&H23, &HD}
        Dim TxBufJRC_frequency() As Byte = {&HD, 0, 0, 0, 0, &HD}
        Dim division, modulo As Integer
        Dim str As String = ""

        If JRCready = False Then
            COMPortJRC.Write(TxBufJRC_connect, 0, 2)
            TxbRx.AppendText("send query to JRL-3000F" & vbCrLf)
        Else
            If RbJRCready.Checked = False Then
                RbJRCready.Checked = True
                RbJRCready.TabStop = False
            End If
            If Me.Frequency <> 0 Then
                If Me.Frequency <> Me.PrevFreq Then
                    division = Me.Frequency \ 1000000
                    modulo = Me.Frequency Mod 1000000
                    TxBufJRC_frequency(2) = (division \ 10) * 16 + (division Mod 10)
                    division = modulo \ 10000
                    modulo = modulo Mod 10000
                    TxBufJRC_frequency(3) = (division \ 10) * 16 + (division Mod 10)
                    division = modulo \ 100
                    modulo = modulo Mod 100
                    TxBufJRC_frequency(4) = (division \ 10) * 16 + (division Mod 10)
                    COMPortJRC.Write(TxBufJRC_frequency, 0, 6)
                    For i As Integer = 0 To 5
                        str &= ByteToHex(TxBufJRC_frequency(i))
                    Next
                    TxbRx.AppendText("send Frequency to JRL-3000F : " & str & vbCrLf)
                    Me.PrevFreq = Me.Frequency
                    LabelJRL3000.Text = Me.Frequency.ToString("#,0") & "Hz"
                End If
            End If
        End If
    End Sub


    Private Sub btnExit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnExit.Click
        Timer1.Enabled = False
        COMportICOM.DiscardInBuffer()
        COMportICOM.Close()
        COMPortJRC.DiscardInBuffer()
        COMPortJRC.Close()
        Me.Close()
    End Sub

    Private Sub btnPause_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles btnPause.Click
        If Timer1.Enabled = True Then
            Timer1.Enabled = False
            TxbRx.Enabled = False
            LabelJRL3000.Enabled = False
        Else
            Timer1.Enabled = True
            TxbRx.Enabled = True
            LabelJRL3000.Enabled = True
        End If
    End Sub
End Class
